(ns blobbox.core
  ;; Lee Spector, lspector@hampshire.edu, 20140409
  (:use quil.core blobbox.globals blobbox.util blobbox.neighbors)
  (:gen-class))

(defn add-blob [b]
  (swap! all-blobs conj b))

(defn setup []
  (smooth)
  (no-stroke)
  (reset! iteration 0)
  #_(add-blob {:position [100 100]
             :velocity [0.5 1.5]
             :rotation 0
             :neighbors []})
  (dotimes [_ 100] (add-blob {:position [(rand-int screen-size) (rand-int screen-size)]
                              :velocity [(* 5 (- (rand) 0.5)) (* 5 (- (rand) 0.5))]
                              :rotation (* two-pi (rand))
                              :neighbors []}))
  
  )

(defn draw-blobs []
  (doseq [b @all-blobs]
    (let [[x y] (:position b)]
      (push-matrix)
      (translate x y)
      (rotate (:rotation b))
      ;; outer membrane
      (fill 128, 255, 0, 100)
      (ellipse 0 0 blob-size blob-size)
      ;; nucleus
      (fill 32, 64, 1, 196) 
      (ellipse 0 0 (/ blob-size 2) (/ blob-size 2))
      ;; heading, relative to 12 o'clock
      (fill 255, 255, 0, 196)
      (ellipse 0 (* blob-size -0.375) (* blob-size 0.25) (* blob-size 0.25))
      (pop-matrix))))

(defn update-blob
  [b]
  (let [neighbors (:neighbors b)
        v (:velocity b)
        new-v (+v v [(- (rand 0.5) 0.25) (- (rand 0.5) 0.25) (- (rand 0.5) 0.25)])]
    (-> b
      ;(assoc :velocity v) ;; unchanged
      (assoc :velocity new-v)
      (assoc :position (+v (:position b) new-v))
      (assoc :rotation (velocity->rotation (nrm new-v))))))

(defn update-blobs
  []
  (swap! all-blobs
         (fn [blobs]
           (map (comp constrain-velocity wrap-position)
                (doall (pmap update-blob blobs))))))

(defn print-stats []
  (println "avg neighbors" (float (/ (reduce + (map count (map :neighbors @all-blobs)))
                                     (count @all-blobs)))))

(defn draw
  []
  (background 175 175 205)
  (swap! iteration inc)
  (update-blobs)
  (draw-blobs)
  ;(update-neighbors) ;;; MEMORY HOLE IN HERE
  ;(print-stats)
  )

#_(defn mouse-moved []
   (let [x (mouse-x)  y (mouse-y)]
     (reset! (state :mouse-position) [x y])))

(defn -main [& args]
  (sketch
    :title "blobbox"
    :setup setup
    :draw draw
    :size [screen-size screen-size]
    ;:mouse-moved mouse-moved
    ;:on-close #(System/exit 0)
    ))

;(-main)
