(ns blobbox.util
  (:use quil.core blobbox.globals))

(def two-pi (* 2 Math/PI))

(defn wrap-angle
  [a]
  (cond (< a 0.0) (- two-pi a)
        (> a two-pi) (- a two-pi)
        :else a))

(defn length [[x y]]
  (sqrt (+ (* x x) (* y y))))

(defn nrm [xy]
  (let [len (length xy)]
    (if (zero? len)
      xy
      (map #(/ % len) xy))))

(defn *v [& vecs-or-nums]
  (apply map * (map #(if (number? %) [% %] %) vecs-or-nums)))

(defn +v [& vecs-or-nums]
  (apply map + (map #(if (number? %) [% %] %) vecs-or-nums)))

(defn -v [& vecs-or-nums]
  (apply map - (map #(if (number? %) [% %] %) vecs-or-nums)))

(defn sign [n] (if (neg? n) -1 1))

(defn wrap ;; to [-1 1]
  [n]
  (* (sign n) (mod (Math/abs n) 1)))

(defn atan2-local ;; http://en.wikipedia.org/wiki/Atan2 ... quil provides atan2 also... the same?
  [y x]
  (cond (> x 0) (Math/atan (/ y x))
        (and (>= y 0) (< x 0)) (+ (Math/atan (/ y x)) Math/PI)
        (and (< y 0) (< x 0)) (- (Math/atan (/ y x)) Math/PI)
        (and (> y 0) (= x 0)) (/ Math/PI 2)
        (and (< y 0) (= x 0)) (- (/ Math/PI 2))
        :else 0))

(defn velocity->rotation
  [[x y]]
  ;;http://stackoverflow.com/questions/2276855/xna-2d-vector-angles-whats-the-correct-way-to-calculate
  (atan2-local x (- y)))
    
(defn constrain-velocity [b]
  (assoc b :velocity [(max (- max-velocity) (min max-velocity (first (:velocity b))))
                      (max (- max-velocity) (min max-velocity (second (:velocity b))))]))

(defn wrap-position [b]
  (let [[x y] (:position b)]
    (assoc b :position [(if (< x 0) 
                          (- screen-size x)
                          (if (>= x screen-size)
                            (- x screen-size)
                            x))
                        (if (< y 0) 
                          (- screen-size y)
                          (if (>= y screen-size)
                            (- y screen-size)
                            y))])))