(ns blobbox.neighbors
  (:use blobbox.globals blobbox.util))

(defn xy->gridxy
  [xy alpha steps step-size]
  (doall (map (fn [coord] 
                (max 0 
                     (min (dec steps) 
                          (int (/ (- coord alpha) step-size)))))
              xy)))

(defn make-world-grid
  [blobs alpha steps step-size]
  (loop [grid (vec (repeat steps (vec (repeat steps []))))
         remaining (map #(dissoc % :neighbors) blobs)]
    (if (empty? remaining)
      grid
      (recur (update-in grid 
                        (xy->gridxy (:position (first remaining)) alpha steps step-size)
                        conj
                        (first remaining))
             (rest remaining)))))

(defn update-neighbors
  []
  (swap! all-blobs
         (fn [blobs]
           (let [alpha (apply min (map #(apply min (:position %)) blobs))
                 omega (apply max (map #(apply max (:position %)) blobs))
                 delta blob-size
                 steps (max 1 (int (/ (- omega alpha) delta)))
                 step-size (max 1 (int (/ (- omega alpha) steps)))
                 world-grid (make-world-grid blobs alpha steps step-size)]
             (doall (map (fn [b]
                           (let [[x y] (xy->gridxy (:position b) alpha steps step-size)
                                 in-range (fn [c] (<= 0 c (dec steps)))
                                 window (fn [c] (map #(+ c %) [-1 0 1]))]
                             (assoc b :neighbors
                                    (map #(dissoc % :neighbors) ;; avoid recursive references
                                         (filter #(< (length (map - 
                                                                  (:position b)
                                                                  (:position %)))
                                                     delta)
                                                 (apply concat
                                                        (doall (for [xs (filter in-range (window x))
                                                                     ys (filter in-range (window y))]
                                                                 (get-in world-grid [xs ys])))))))))
                         blobs))))))
